@@ -79,7 +79,7 @@ typedef enum {
 	EXTRACT_MIME_AUDIO,
 	EXTRACT_MIME_VIDEO,
 	EXTRACT_MIME_IMAGE,
-	EXTRACT_MIME_3GPP,
+	EXTRACT_MIME_GUESS,
 } ExtractMime;
 
 typedef struct {
@@ -124,7 +124,7 @@ static void extract_gstreamer_video (const gchar          *uri,
 static void extract_gstreamer_image (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
-static void extract_gstreamer_3gpp  (const gchar          *uri,
+static void extract_gstreamer_guess (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 
@@ -132,8 +132,9 @@ static TrackerExtractData data[] = {
 	{ "audio/*", extract_gstreamer_audio },
 	{ "video/*", extract_gstreamer_video },
 	{ "image/*", extract_gstreamer_image },
-	/* mime type guessing returns video/3gpp also for 3gpp audio files */
-	{ "video/3gpp", extract_gstreamer_3gpp },
+	/* mime type guessing returns for instance video/3gpp also for 3gpp audio files */
+	{ "video/3gpp", extract_gstreamer_guess },
+	{ "video/mp4", extract_gstreamer_guess },
 	{ NULL, NULL }
 };
 
@@ -324,6 +325,32 @@ add_time_gst_tag (TrackerSparqlBuilder   *metadata,
 	}
 }
 
+static void
+add_keywords_gst_tag (TrackerSparqlBuilder *metadata,
+                      GstTagList           *tag_list)
+{
+	gboolean ret;
+	gchar *str;
+
+	ret = gst_tag_list_get_string (tag_list, GST_TAG_KEYWORDS, &str);
+
+	if (ret) {
+		GStrv keywords;
+		gint i = 0;
+
+		keywords = g_strsplit_set (str, " ,", -1);
+
+		while (keywords[i]) {
+			tracker_sparql_builder_predicate (metadata, "nie:keyword");
+			tracker_sparql_builder_object_unvalidated (metadata, g_strstrip (keywords[i]));
+			i++;
+		}
+
+		g_strfreev (keywords);
+		g_free (str);
+	}
+}
+
 static gboolean
 get_embedded_album_art(MetadataExtractor *extractor)
 {
@@ -471,11 +498,12 @@ extract_metadata (MetadataExtractor      *extractor,
 	g_return_if_fail (metadata != NULL);
 
 	if (extractor->tagcache) {
+		gchar *artist_uri = NULL;
 		gchar *performer_uri = NULL;
 		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
 
-		if (extractor->mime == EXTRACT_MIME_3GPP) {
+		if (extractor->mime == EXTRACT_MIME_GUESS) {
 			gchar *video_codec = NULL, *audio_codec = NULL;
 
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_VIDEO_CODEC, &video_codec);
@@ -499,6 +527,18 @@ extract_metadata (MetadataExtractor      *extractor,
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_PERFORMER, &performer);
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ARTIST, &artist_local);
 
+			if (artist_local) {
+				artist_uri = tracker_uri_printf_escaped ("urn:artist:%s", artist_local);
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, artist_uri);
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nmm:Artist");
+				tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
+				tracker_sparql_builder_object_unvalidated (preupdate, artist_local);
+				tracker_sparql_builder_insert_close (preupdate);
+			}
+
 			s = tracker_coalesce (2, performer, artist_local);
 
 			if (s) {
@@ -557,10 +597,15 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_predicate (preupdate, "nmm:albumTitle");
 			tracker_sparql_builder_object_unvalidated (preupdate, s);
 
+			if (artist_uri) {
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumArtist");
+				tracker_sparql_builder_object_iri (preupdate, artist_uri);
+			}
+
 			tracker_sparql_builder_insert_close (preupdate);
 
-			has_it = gst_tag_list_get_uint (extractor->tagcache, 
-			                                GST_TAG_TRACK_COUNT, 
+			has_it = gst_tag_list_get_uint (extractor->tagcache,
+			                                GST_TAG_TRACK_COUNT,
 			                                &count);
 
 			if (has_it) {
@@ -585,8 +630,8 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
-			has_it = gst_tag_list_get_uint (extractor->tagcache, 
-			                                GST_TAG_ALBUM_VOLUME_NUMBER, 
+			has_it = gst_tag_list_get_uint (extractor->tagcache,
+			                                GST_TAG_ALBUM_VOLUME_NUMBER,
 			                                &count);
 
 			if (has_it) {
@@ -611,8 +656,8 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
-			has_it = gst_tag_list_get_double (extractor->tagcache, 
-			                                  GST_TAG_ALBUM_GAIN, 
+			has_it = gst_tag_list_get_double (extractor->tagcache,
+			                                  GST_TAG_ALBUM_GAIN,
 			                                  &gain);
 
 			if (has_it) {
@@ -637,8 +682,8 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
-			has_it = gst_tag_list_get_double (extractor->tagcache, 
-			                                  GST_TAG_ALBUM_PEAK, 
+			has_it = gst_tag_list_get_double (extractor->tagcache,
+			                                  GST_TAG_ALBUM_PEAK,
 			                                  &gain);
 
 			if (has_it) {
@@ -675,7 +720,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		if (needs_audio) {
 			tracker_sparql_builder_object (metadata, "nmm:MusicPiece");
 			tracker_sparql_builder_object (metadata, "nfo:Audio");
-		} 
+		}
 
 		if (extractor->mime == EXTRACT_MIME_VIDEO) {
 			tracker_sparql_builder_object (metadata, "nmm:Video");
@@ -715,6 +760,8 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_predicate (metadata, "nmm:director");
 				tracker_sparql_builder_object_iri (metadata, composer_uri);
 			}
+
+			add_keywords_gst_tag (metadata, extractor->tagcache);
 		}
 
 		if (needs_audio) {
@@ -746,6 +793,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		}
 
 		g_free (performer_uri);
+		g_free (artist_uri);
 		g_free (composer_uri);
 		g_free (album_uri);
 
@@ -759,7 +807,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		if (needs_audio) {
 			tracker_sparql_builder_object (metadata, "nmm:MusicPiece");
 			tracker_sparql_builder_object (metadata, "nfo:Audio");
-		} 
+		}
 
 		if (extractor->mime == EXTRACT_MIME_VIDEO) {
 			tracker_sparql_builder_object (metadata, "nmm:Video");
@@ -1195,11 +1243,11 @@ extract_gstreamer_image (const gchar          *uri,
 }
 
 static void
-extract_gstreamer_3gpp (const gchar          *uri,
+extract_gstreamer_guess (const gchar          *uri,
                         TrackerSparqlBuilder *preupdate,
 			TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_3GPP);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_GUESS);
 }
 
 TrackerExtractData *
